Index: src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java
===================================================================
--- src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java	(revision 2586)
+++ src/uk/me/parabola/mkgmap/osmstyle/StyledConverter.java	(working copy)
@@ -108,6 +108,8 @@
 	
 	private List<Way> roads = new ArrayList<Way>();
 	private List<GType> roadTypes = new ArrayList<GType>();
+	private List<Way> lines = new ArrayList<Way>();
+	private List<GType> lineTypes = new ArrayList<GType>();
 	private double minimumArcLength;
 	
 	private int nextNodeId = 1;
@@ -240,8 +242,10 @@
 		    	roads.add(way);
 		    	roadTypes.add(new GType(foundType));
 		    }
-		    else
-				addLine(way, foundType);
+		    else {
+		    	lines.add(way);
+		    	lineTypes.add(new GType(foundType));
+		    }
 		}
 		else
 			addShape(way, foundType);
@@ -330,6 +334,15 @@
 		setHighwayCounts();
 		findUnconnectedRoads();
 		removeShortArcsByMergingNodes(minimumArcLength);
+		for (int i = 0; i < lines.size(); i++){
+			Way line = lines.get(i);
+			if (line == null)
+				continue;
+			GType gt = lineTypes.get(i);
+			addLine(line, gt);
+		}
+		lines = null;
+		lineTypes = null;
 		for (int i = 0; i < roads.size(); i++){
 			Way road = roads.get(i);
 			if (road == null)
@@ -1849,6 +1862,8 @@
 	private void removeShortArcsByMergingNodes(double minArcLength) {
 		log.info("Removing short arcs (min arc length = " + minArcLength + "m)");
 		log.info("Removing short arcs - marking points as node-alike");
+		HashMap<Long, Way> modified = new HashMap<Long, Way>();
+		HashSet<Long> deleted = new HashSet<Long>();
 		for (Way way : roads) {
 			if (way == null)
 				continue;
@@ -1894,6 +1909,7 @@
 					if (log.isInfoEnabled())
 						log.info("  Way " + way.getTag("name") + " (" + way.toBrowseURL() + ") has less than 2 points - deleting it");
 					roads.set(w, null);
+					deleted.add(way.getId());
 					++numWaysDeleted;
 					continue;
 				}
@@ -1923,6 +1939,7 @@
 							points.set(i, p);
 							if (i == 0)
 								previousPoint = p;
+							modified.put(way.getId(), way);
 							anotherPassRequired = true;
 						}
 					}
@@ -1938,6 +1955,7 @@
 						points.remove(i);
 						// hack alert! rewind the loop index
 						--i;
+						modified.put(way.getId(), way);
 						anotherPassRequired = true;
 						continue;
 					}
@@ -2058,11 +2076,25 @@
 
 					// hack alert! rewind the loop index
 					i = previousNodeIndex;
+					modified.put(way.getId(), way);
 					anotherPassRequired = true;
 				}
 			}
 		}
-
+		// make sure that copies of modified roads are have equal points 
+		for (int i = 0; i < lines.size(); i++){
+			Way line = lines.get(i);
+			if (deleted.contains(line.getId())){
+				lines.set(i, null);
+				continue;
+			}
+			Way modWay = modified.get(line.getId());
+			if (modWay != null){
+				List<Coord> points = line.getPoints();
+				points.clear();
+				points.addAll(modWay.getPoints());
+			}
+		}
 		if (anotherPassRequired)
 			log.error("Removing short arcs - didn't finish in " + pass + " passes, giving up!");
 		else
